# Author: Jan Vcelak <jvcelak@redhat.com>

class CarrierFactory
	def initialize(remote_sources_root)
		@remote_sources_root = remote_sources_root
	end

	def build_local(source_dir)
		LocalCarrier.new source_dir
	end

	def build_remote(user, host, identity_file, sources_dir, task_name)
		RemoteCarrier.new user, host, identity_file, @remote_sources_root, task_name
	end
end

require "fileutils"

class LocalCarrier
	def initialize(source_dir)
		@source_dir = source_dir
	end

	def get(target_dir, source_files)
		unless File.directory? target_dir
			raise "Target dir is not a directory"
		end

		source_files.each do |filename|
			filename_base = File.basename filename
			full_source = File.join @source_dir, filename
			full_target = File.join target_dir, filename_base

			FileUtils.cp_r full_source, full_target
		end
	end
end

class RemoteCarrier
	def initialize(user, host, identity_file, sources_root, task_name)
		@user = user
		@host = host
		@identity_file = identity_file
		@sources_root = sources_root
		@task_name = task_name
	end

	def get(target_dir, source_files)
		system *command(target_dir, source_files)
	end

	private

	def command(target_dir, source_files)
		[
			"/usr/bin/scp",
			"-pr", # preserve attributes, recursive
			"-o", "BatchMode=yes",
			"-o", "StrictHostKeyChecking=no",
			"-i", @identity_file,
			sources(source_files),
			target_dir
		]
	end

	def sources(source_files)
		sources = "#{@user}@#{@host}:#{@sources_root}/#{@task_name}/"
		if source_files.size == 1
			sources += source_files[0]
		else
			sources += "{%s}" % source_files.join(",")
		end
		sources
	end
end
