# Author: Jan Vcelak <jvcelak@redhat.com>

require "timeout"

class Exec
	def initialize(user, exec_dir, limits, log, check_logfile)
		@user = user
		@exec_dir = exec_dir
		@limits = limits
		@log = log
		@check_logfile = check_logfile
	end

	def run(command)
		result = run_restricted command
		@log.msg "test result: %s" % result
		result
	end

	def interrupt
		# FIXME: not tested, thread safety?
		kill_remaining
	end

	private

	STATUS_EXEC_ERROR = 250

	def build_command(command)
		ulimit_cmd = @limits.ulimit_command

		cmds = []
		cmds << 'cd "%s"' % @exec_dir
		cmds << ulimit_cmd unless ulimit_cmd.nil?
		cmds << "./#{command}"

		# FIXME: Daniel Walsh wrote about sudo vs runuser...
		["/usr/bin/sudo", "-u", @user, "/bin/sh", "-x", "-c", "{ %s; } >> #{@check_logfile} 2>&1" % cmds.join("; ")]
	end

	def kill_remaining
		@log.msg "killing remaining processes"
		system *["/usr/bin/pkill", "-KILL", "-u", @user]
		# 0 - at least one process matched, 1 - no process matched
		[0, 1].include? $?
	end

	def run_restricted(script)
		command = build_command script
		exit_status = nil
		begin
			@log.cmd command
			@log.msg "timeout: %d seconds" % @limits.time
			Timeout.timeout(@limits.time) do
				exit_status = run_forked command
			end
		rescue Timeout::Error
			@log.msg "execution timeout"
			exit_status = :timeout
		ensure
			kill_remaining
		end

		exit_status
	end

	def run_forked(command)
		cmd_pid = Process.fork do
			begin
				#@log.cmd command
				exec *command
			rescue Exception => exception
				@log.msg exception.message, exception.backtrace
				exit STATUS_EXEC_ERROR
			end
		end

		exit_info = Process.waitpid2(cmd_pid)[1]
		exit_status = exit_info.exitstatus

		@log.msg "finished, exit status %d" % exit_status
		if exit_status == STATUS_EXEC_ERROR
			@log.msg "exec() error"
			exit_status = :error
		end

		exit_status
	end

	def system(*params)
		@log.cmd params
		Kernel.system *params
	end
end
