# Author: Jan Vcelak <jvcelak@redhat.com>

require "bughunting/check/task"
require "bughunting/common"
require "bughunting/server/config"
require "fileutils"
require "shellwords"
require "xmlrpc/client"

class DevelopmentController
	def initialize(task_dir, server_config)
		task_dir = File.expand_path task_dir

		@server_config = ServerConfig.load_file server_config
		@task = Task.new task_dir

		@xmlrpc = XMLRPC::Client.new3({ "host" => "::1", "port" => @server_config["listen.port"] })
	end

	def check
		begin
			puts "Requesting evaluation"
			result = @xmlrpc.call("bughunting.local_check", @task.dir, @task.work_dir)
		rescue Errno::ECONNREFUSED
			puts "Cannot connect to the testing server."
			return false
		end

		(success, message) = result

		puts "Result: %s, %s" % [success ? "success".green.bold : "failure".red.bold, message]
		return success
	end

	def setup
		if File.exists? @task.cache_dir
			puts "Source codes are already set up."
			clean
		else
			system "cp", "-ar", @task.source_dir, @task.work_dir
			run_setup_commands
			create_cache
		end
	end

	def clean
		if test("d", @task.cache_dir)
			system "mkdir", "-p", @task.work_dir unless test("d", @task.work_dir)
			system "rsync", "-a", "--del", @task.cache_dir + "/", @task.work_dir
		end
	end

	def superclean
		system "rm", "-rf", @task.work_dir
		unless @task.symlinked_source
			system "rm", "-rf", @task.cache_dir
		end
	end
	
	def patch
		system "diff", "-uNPrp", @task.cache_dir, @task.work_dir
	end

	private

	def run_setup_commands
		commands = [ "cd '#{@task.work_dir}'" ]
		commands += @task.config["setup_commands"]
		system "sh", "-c", commands.join(";")
	end

	def create_cache
		system "cp", "-ar", @task.work_dir, @task.cache_dir
	end

	def system(*command)
		$stderr.puts "+ " + Shellwords.join(command)
		Kernel.system *command
		$?
	end
end
