# Author: Jan Vcelak <jvcelak@redhat.com>

require "xmlrpc/server"

class Server < XMLRPC::Server
	def initialize(config, service, local_mode = false)
		port = config["listen.port"]

		if not local_mode
			ip = config["listen.ip"]
			max_connections = config["users"].count
		else
			ip = "::1"
			max_connections = 1
		end

		super(port, ip, max_connections, $stderr)
		add_introspection()
		add_handler "bughunting", service
	end

	def service(request, response)
		if @valid_ip 
			raise WEBrick::HTTPStatus::Forbidden unless @valid_ip.any? { |ip| request.peeraddr[3] =~ ip }
		end

		if request.request_method != "POST"
			raise WEBrick::HTTPStatus::MethodNotAllowed,
				"unsupported method `#{request.request_method}'."
		end

		if parse_content_type(request['Content-type']).first != "text/xml" 
			raise WEBrick::HTTPStatus::BadRequest
		end 

		length = (request['Content-length'] || 0).to_i

		raise WEBrick::HTTPStatus::LengthRequired unless length > 0

		data = request.body

		if data.nil? or data.size != length
			raise WEBrick::HTTPStatus::BadRequest
		end

		resp = process(data, request)
		if resp.nil? or resp.size <= 0  
			raise WEBrick::HTTPStatus::InternalServerError
		end

		response.status = 200
		response['Content-Length'] = resp.size
		response['Content-Type']   = "text/xml; charset=utf-8"
		response.body = resp 
	end

	def process(data, request)
		method, params = parser().parseMethodCall(data)
		if method.start_with? "bughunting."
			params.unshift request
		end
		handle(method, *params)
	end
end
