# Author: Jan Vcelak <jvcelak@redhat.com>

require "bughunting/common"

require "thread"

class Users
	def initialize(users = [])
		@users = users.clone
		@free_users = @users.clone
		@running_users = []

		@mutex = Mutex.new
	end

	def get_free(ip)
		user = nil
		@mutex.synchronize do
			next if has_running_test? ip
			next if @free_users.empty?
			user = add_running(ip)
		end
		user
	end

	def return(ip)
		@mutex.synchronize do
			remove_running(ip)
		end
	end

	private

	def has_running_test?(ip)
		@running_users.each do |running|
			return true if running[:ip] == ip
		end
		false
	end

	def add_running(ip)
		reservation = {
			:ip   => ip,
			:user => @free_users.shift
		}
		@running_users.push reservation
		reservation[:user]
	end

	def remove_running(ip)
		user = nil
		@running_users.each_with_index do |reservation, index|
			if reservation[:ip] == ip
				user = reservation[:user]
				@running_users.delete_at index
				break
			end
		end

		unless user.nil?
			@free_users.unshift user
		end
	end
end

